{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Edit IPsec site connection" %}{% endblock %}

{% block main %}
  {% include 'project/vpn/_update_ipsecsiteconnection.html' %}
{% endblock %}
